/*
  kxchanger.cc
  By Wayne Piekarski - wayne@cs.unisa.edu.au, wayne@tinmith.net
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  See the file LICENSE included in this directory for more information.
*/


#include <qapplication.h>
#include <qmainwindow.h>
#include <qpushbutton.h>
#include <qlcdnumber.h>
#include <qhgroupbox.h>
#include <qvgroupbox.h>
#include <qtimer.h>
#include <qlabel.h>
#include <qgrid.h>
#include <qhbox.h>
#include <qvbox.h>

#include <stdlib.h>
#include <stdio.h>




#define CONFIG_FILE "/etc/xchanger.conf"
#define BUFFER_SIZE 1024




/* This is used to receive button presses */
class ButtonHandler : public QObject
{
  Q_OBJECT
  
public:
  /* Constructor */
  ButtonHandler (void);
  
  
  /* Slot methods to receive the buttons */
public slots:
  
  void pri1400x1050 (void) { setPrimaryResolution ("1400x1050"); };
  void pri1280x1024 (void) { setPrimaryResolution ("1280x1024"); };
  void pri1024x768  (void) { setPrimaryResolution ("1024x768"); };
  void pri800x600   (void) { setPrimaryResolution ("800x600"); };
  void pri640x480   (void) { setPrimaryResolution ("640x480"); };
  
  void sec1400x1050 (void) { setSecondaryResolution ("1400x1050"); };
  void sec1280x1024 (void) { setSecondaryResolution ("1280x1024"); };
  void sec1024x768  (void) { setSecondaryResolution ("1024x768"); };
  void sec800x600   (void) { setSecondaryResolution ("800x600"); };
  void sec640x480   (void) { setSecondaryResolution ("640x480"); };
  
  void monDUALCRT   (void) { setDisplayLayout ("DUALHEAD"); setSecondaryOutput ("CRT"); };
  void monCLONECRT  (void) { setDisplayLayout ("CLONE");    setSecondaryOutput ("CRT"); };
  void monSINGLE    (void) { setDisplayLayout ("SINGLE");   setSecondaryOutput ("CRT"); };
  void monDUALTV    (void) { setDisplayLayout ("DUALHEAD"); setSecondaryOutput ("TV");  };
  void monCLONETV   (void) { setDisplayLayout ("CLONE");    setSecondaryOutput ("TV");  };
  
  void inpPS2       (void) { setInputMouse ("PS2"); };
  void inpUSB       (void) { setInputMouse ("USB"); };
  void inpBOTH      (void) { setInputMouse ("BOTH"); };
  
  void conQUIT (void);
  void conSAVE (void);
  
  void setPrimaryResolution (char *in);
  void setSecondaryResolution (char *in);
  void setSecondaryOutput (char *in);
  void setDisplayLayout (char *in);
  void setInputMouse (char *in);
  
  void refresh (void);
  
  
public:
  
  /* Global values which we need to reference */
  QLabel *summary;
  char primary_resolution   [BUFFER_SIZE];
  char secondary_resolution [BUFFER_SIZE];
  char secondary_output     [BUFFER_SIZE];
  char display_layout       [BUFFER_SIZE];
  char input_mouse          [BUFFER_SIZE];
};




#include "kxchanger.moc"




void ButtonHandler::setPrimaryResolution (char *in)
{
  strcpy (primary_resolution, in);
  refresh ();
}




void ButtonHandler::setSecondaryResolution (char *in)
{
  strcpy (secondary_resolution, in);
  refresh ();
}




void ButtonHandler::setSecondaryOutput (char *in)
{
  strcpy (secondary_output, in);
  refresh ();
}




void ButtonHandler::setDisplayLayout (char *in)
{
  strcpy (display_layout, in);
  refresh ();
}




void ButtonHandler::setInputMouse (char *in)
{
  strcpy (input_mouse, in);
  refresh ();
}




int copy_helper (char *storage, char *check, char *incoming)
{
  if (strstr (incoming, check) == incoming)
    {
      strcpy (storage, incoming + strlen (check));
      return (1);
    }
  else
    return (0);
}




ButtonHandler::ButtonHandler (void)
{
  /* Parse in the configuration file */
  FILE *fp = fopen (CONFIG_FILE, "r");
  if (fp == NULL)
    {
      fprintf (stderr, "Couldn't open %s file for reading - using defaults\n", CONFIG_FILE);
      strcpy (primary_resolution,   "800x600");
      strcpy (secondary_resolution, "800x600");
      strcpy (secondary_output,     "CRT");
      strcpy (display_layout,       "SINGLE");
      strcpy (input_mouse,          "BOTH");
      return;
    }
  
  /* Read each line */
  char buffer [BUFFER_SIZE];
  while (fgets (buffer, BUFFER_SIZE, fp) != NULL)
    {
      /* Strip off any new lines */
      char *nl = buffer;
      while ((*nl != '\0') && (*nl != '\n'))
	nl++;
      *nl = '\0';
      
      /* Copy in the value as needed */
      int result = 0;
      result += copy_helper (primary_resolution, "PRIMARY_RESOLUTION=", buffer);
      result += copy_helper (secondary_resolution, "SECONDARY_RESOLUTION=", buffer);
      result += copy_helper (secondary_output, "SECONDARY_OUTPUT=", buffer);
      result += copy_helper (display_layout, "DISPLAY_LAYOUT=", buffer);
      result += copy_helper (input_mouse, "INPUT_MOUSE=", buffer);
      
      /* Error checking */
      if (result == 0)
	fprintf (stderr, "Skipping line with invalid data [%s]\n", buffer);
      else if (result > 1)
	{
	  fprintf (stderr, "The input data [%s] matched more than one rule, which should be impossible", buffer);
	  exit (1);
	}
    }
}




void ButtonHandler::conQUIT (void)
{
  exit (0);
}




void ButtonHandler::conSAVE (void)
{
  /* Write out the config file */
  FILE *fp = fopen (CONFIG_FILE, "w");
  if (fp == NULL)
    {
      fprintf (stderr, "Couldn't open %s file for writing\n", CONFIG_FILE);
      exit (1);
    }
  fprintf (fp, "PRIMARY_RESOLUTION=%s\n", primary_resolution);
  fprintf (fp, "SECONDARY_RESOLUTION=%s\n", secondary_resolution);
  fprintf (fp, "SECONDARY_OUTPUT=%s\n", secondary_output);
  fprintf (fp, "DISPLAY_LAYOUT=%s\n", display_layout);
  fprintf (fp, "INPUT_MOUSE=%s\n", input_mouse);
  
  fclose (fp);
  
  /* Regenerate the configuration file */
  system ("xchcc");
  
  /* Done */
  exit (0);
}




void ButtonHandler::refresh (void)
{
  char buffer [BUFFER_SIZE];
  sprintf (buffer, "Main='%s' 2ndOutput='%s' 2nd='%s' Layout='%s' Mouse='%s'",
	   primary_resolution, secondary_output, secondary_resolution, display_layout, input_mouse);
  summary->setText (buffer);
}




int main (int argc, char *argv[])
{
  /* Initialise the application */
  QApplication app (argc, (char **)argv);
  
  /* Allocate a button handler */
  ButtonHandler *buttonhandler = new ButtonHandler ();
  
  /* Create the application window, a top level box, and make it the main widget */
  QMainWindow *window = new QMainWindow (NULL, "KXchanger");
  QVGroupBox *topbox = new QVGroupBox (window);
  window->setCentralWidget (topbox);
  
  /* Add a title label */
  new QLabel ("<B><FONT SIZE=+2>KXchanger - By Wayne Piekarski</FONT></B>", topbox);
  
  /* Add three columns to put the parts of the app into it */
  QHGroupBox *qhbox = new QHGroupBox (topbox);
  
  /* Create the primary resolution items now */
  {
    QVGroupBox *qvboxtop = new QVGroupBox (qhbox);
    
    new QLabel ("<B>Primary Resolution</B>", qvboxtop);
    QVGroupBox *qvbox = new QVGroupBox (qvboxtop);
    
    QPushButton *pri1400x1050 = new QPushButton ("1400x1050", qvbox);
    QObject::connect (pri1400x1050, SIGNAL(clicked()), buttonhandler, SLOT(pri1400x1050()));

    QPushButton *pri1280x1024 = new QPushButton ("1280x1024", qvbox);
    QObject::connect (pri1280x1024, SIGNAL(clicked()), buttonhandler, SLOT(pri1280x1024()));

    QPushButton *pri1024x768 = new QPushButton ("1024x768", qvbox);
    QObject::connect (pri1024x768, SIGNAL(clicked()), buttonhandler, SLOT(pri1024x768()));

    QPushButton *pri800x600 = new QPushButton ("800x600", qvbox);
    QObject::connect (pri800x600, SIGNAL(clicked()), buttonhandler, SLOT(pri800x600()));
  }
  
  /* Create the monitors controls now */
  {
    QVGroupBox *qvboxtop = new QVGroupBox (qhbox);
    
    new QLabel ("<B>Monitors</B>", qvboxtop);
    QVGroupBox *qvbox = new QVGroupBox (qvboxtop);
    
    QPushButton *monDUALCRT = new QPushButton ("Display + CRT (Dual Head)", qvbox);
    QObject::connect (monDUALCRT, SIGNAL(clicked()), buttonhandler, SLOT(monDUALCRT()));
    
    QPushButton *monCLONECRT = new QPushButton ("Display + CRT (Cloned)", qvbox);
    QObject::connect (monCLONECRT, SIGNAL(clicked()), buttonhandler, SLOT(monCLONECRT()));
    
    QPushButton *monDUALTV = new QPushButton ("Display + TV (Dual Head)", qvbox);
    QObject::connect (monDUALTV, SIGNAL(clicked()), buttonhandler, SLOT(monDUALTV()));
    
    QPushButton *monCLONETV = new QPushButton ("Display + TV (Cloned)", qvbox);
    QObject::connect (monCLONETV, SIGNAL(clicked()), buttonhandler, SLOT(monCLONETV()));
    
    QPushButton *monSINGLE = new QPushButton ("Display only (Single)", qvbox);
    QObject::connect (monSINGLE, SIGNAL(clicked()), buttonhandler, SLOT(monSINGLE()));
  }
  
  /* Create the secondary resolution items now */
  {
    QVGroupBox *qvboxtop = new QVGroupBox (qhbox);
    
    new QLabel ("<B>Secondary Resolution</B>", qvboxtop);
    QVGroupBox *qvbox = new QVGroupBox (qvboxtop);
    
    QPushButton *sec1400x1050 = new QPushButton ("1400x1050", qvbox);
    QObject::connect (sec1400x1050, SIGNAL(clicked()), buttonhandler, SLOT(sec1400x1050()));

    QPushButton *sec1280x1024 = new QPushButton ("1280x1024", qvbox);
    QObject::connect (sec1280x1024, SIGNAL(clicked()), buttonhandler, SLOT(sec1280x1024()));

    QPushButton *sec1024x768 = new QPushButton ("1024x768", qvbox);
    QObject::connect (sec1024x768, SIGNAL(clicked()), buttonhandler, SLOT(sec1024x768()));

    QPushButton *sec800x600 = new QPushButton ("800x600", qvbox);
    QObject::connect (sec800x600, SIGNAL(clicked()), buttonhandler, SLOT(sec800x600()));
  }
  
  /* Create the mouse controls now */
  {
    QVGroupBox *qvboxtop = new QVGroupBox (qhbox);
    
    new QLabel ("<B>Mouse Input</B>", qvboxtop);
    QVGroupBox *qvbox = new QVGroupBox (qvboxtop);
    
    QPushButton *inpUSB = new QPushButton ("/dev/input/mice (For 2.6)", qvbox);
    QObject::connect (inpUSB, SIGNAL(clicked()), buttonhandler, SLOT(inpUSB()));
    
    QPushButton *inpBOTH = new QPushButton ("Both (For 2.4)", qvbox);
    QObject::connect (inpBOTH, SIGNAL(clicked()), buttonhandler, SLOT(inpBOTH()));
    
    QPushButton *inpPS2 = new QPushButton ("/dev/psaux (For 2.4)", qvbox);
    QObject::connect (inpPS2, SIGNAL(clicked()), buttonhandler, SLOT(inpPS2()));
  }
  
  /* Create the misc controls now */
  {
    QVGroupBox *qvbox = new QVGroupBox (qhbox);
    new QLabel ("<B>Controls</B>", qvbox);
    
    QPushButton *conQUIT = new QPushButton ("Quit", qvbox);
    QObject::connect (conQUIT, SIGNAL(clicked()), buttonhandler, SLOT(conQUIT()));
    
    QPushButton *conSAVE = new QPushButton ("Save", qvbox);
    QObject::connect (conSAVE, SIGNAL(clicked()), buttonhandler, SLOT(conSAVE()));
  }
  
  /* Add a summary label with the final options */
  buttonhandler->summary = new QLabel ("Options not available yet\n", topbox);
  
  /* Refresh the widgets */
  buttonhandler->refresh ();
  
  /* Finalise the application and run it */
  app.setMainWidget (window);
  window->show ();
  app.exec ();
  
  /* Never executed */
  exit (1);
}
